"""
This code handles the Raspberry Pi LCD display by writing directly
to the framebuffer

The map image is in the images subdirectory and the font
is in the fonts subdirectory.

Concept, Design and Implementation by: Craig A. Lindley
Last Update: 10/07/2021
Version: 1.0
"""

import os
os.environ['PYGAME_HIDE_SUPPORT_PROMPT'] = "hide" # hide pygame prompt message
import pygame
import pygame.freetype  # Import the freetype module.
import time
import sys
from datetime import datetime

from pygame.locals import *

class DisplayManager:

	# Class constructor
	def __init__(self):
		self.topTextRow = 12
		self.eventsTextRow = 415
		self.bottomTextRow = 455
		self.fontSize = 40
		self.textColor = (255, 255, 255)
		self.black  = (0, 0, 0)
		self.white  = (255, 255, 255)
		self.red    = (255, 0, 0)
		self.yellow = (255, 255, 0)
		self.green  = (0, 255, 0)
		self.blue  = (0, 0, 255)

		pygame.init()
		pygame.mouse.set_visible(0)
		self.screenWidth  = pygame.display.Info().current_w
		self.screenHeight = pygame.display.Info().current_h
		size = (self.screenWidth, self.screenHeight)
		self.screen = pygame.display.set_mode(size)

		# Read the make into memory
		self.mapImage = pygame.image.load('images/eqm800.bmp')

		# Get its bounding box
		self.mapImageRect = self.mapImage.get_rect()

		# Center map vertically
		self.mapImageRect.y = (self.screenHeight - self.mapImageRect.height) / 2;

		# Setup inital font of initial size
		self.font = pygame.freetype.Font('fonts/Sony.ttf', self.fontSize)

        # Clear the screen
	def clearScreen(self):
		self.screen.fill(self.black)
		pygame.display.flip()

	# Backlight control
	def backlight(self, b):
		if b:
			# Turn backlight on
			os.system("sudo sh -c 'echo 0 > /sys/class/backlight/rpi_backlight/bl_power'")
		else:
			# Turn backlight off
			os.system("sudo sh -c 'echo 1 > /sys/class/backlight/rpi_backlight/bl_power'")

	# Select color from magnitude
	def colorFromMag(self, mag):
		if mag < 1:
			mag = 1.0

		imag = int(mag + 0.5)
		case = {
			1: self.green,
			2: self.green,
			3: self.green,
			4: self.yellow,
			5: self.yellow,
			6: self.yellow,
			7: self.red,
			8: self.red,
			9: self.red
		}
		return case.get(imag, self.green)

	# Display the map
	def displayMap(self):
		self.clearScreen()
		self.screen.blit(self.mapImage, self.mapImageRect)
		pygame.display.flip()

	# Draw text
	def drawText(self, x, y, text):
		self.font.render_to(self.screen, (x, y), text, self.textColor)
		pygame.display.flip()

	# Draw centered text
	def drawCenteredText(self, y, text):
		textSurface, rect = self.font.render(text, 1, self.textColor)
		x = (self.screenWidth - rect.width) / 2
		self.font.render_to(self.screen, (x, y), text, self.textColor)
		pygame.display.flip()

	# Draw right justified text
	def drawRightJustifiedText(self, y, text):
                textSurface, rect = self.font.render(text, 1, self.textColor)
                x = self.screenWidth - rect.width - 1
                self.font.render_to(self.screen, (x, y), text, self.textColor)
                pygame.display.flip()

	# Set text size
	def setTextSize(self, size):
		self.fontSize = size
		self.font = pygame.freetype.Font('fonts/Sony.ttf', self.fontSize)

	# Set text color
	def setTextColor(self, color):
		self.textColor = color

	# Draw a circle on the scrren
	def drawCircle(self, x, y, radius, color):
		pygame.draw.circle(self.screen, color, (x, y), radius, width=2)
		pygame.display.flip()

	# Draw a circle with size based on mag at lon, lat position on map
	def mapEarthquake(self, lon, lat, mag, color):
		# Calculate map X and Y
		mapX = ((lon + 180.0) * self.mapImageRect.width) / 360.0
		mapY = ((((-1 * lat) + 90.0) * self.mapImageRect.height) / 180.0) + self.mapImageRect.y

		# Determine circle radius from mag
		if mag < 2:
			mag = 2.0
		radius = mag * 3

		# Draw a circle at earthquake location
		self.drawCircle(mapX, mapY, radius, color);

	# Display current time
	def displayCurrentTime(self):
		now = datetime.now()
		timeString = now.strftime("%-I:%M %P")
		self.drawText(0, self.topTextRow, "Time: " + timeString)

	# Display magnitude
	def displayMagnitude(self, mag):
		self.drawCenteredText(self.topTextRow, "Mag: " + str(mag))

	# Display depth
	def displayDepth(self, depth):
		# Convert kilometers to miles
		miles = depth / 1.609344
		milesStr = "Depth: {d:.2f} m"
		self.drawRightJustifiedText(self.topTextRow, milesStr.format(d = miles))

	# Display number of events in DB
	def displayNumberOfEvents(self, num):
		self.setTextColor(self.blue)
		self.setTextSize(20)
		self.drawCenteredText(self.eventsTextRow, "EQ Events: " + str(num))
		self.setTextColor(self.white)
		self.setTextSize(40)

	# Display location with color from magnitude
	def displayLocation(self, location, mag):
		self.setTextColor(self.colorFromMag(mag))
		self.drawCenteredText(self.bottomTextRow, location)
		self.setTextColor(self.white)

	# Display title page
	def displayTitlePage(self):
		self.displayMap()
		self.drawCenteredText(90, "Realtime")
		self.setTextSize(70)
		self.drawCenteredText(160, "World Earthquake Map")
		self.setTextSize(40)
		self.drawText(0, 350, " Sept 2021")
		self.drawRightJustifiedText(350, "Craig A. Lindley ")
		time.sleep(10)


# Create global instance
displayManager = DisplayManager()

"""
# Test Code
import time

#displayManager.displayTitlePage()

displayManager.displayMap()
time.sleep(5)
displayManager.backlight(False)
time.sleep(5)
displayManager.backlight(True)

displayManager.drawText(0, 240, "This is some text")
displayManager.drawRightJustifiedText( 240, "This is some text")

displayManager.drawCenteredText(100, "This is some text to display")

displayManager.drawCircle(400, 240, 40, (0,0,255))
displayManager.mapEarthquake(0, 0, 2, (0,255,255))

displayManager.displayCurrentTime()
displayManager.displayMagnitude(3.6)
displayManager.displayDepth(-3.6)
displayManager.displayLocation("Tip of south Africa", 6.5)

while True:

	time.sleep(20)
"""



